// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:async';
import 'dart:html';

import '../../../src/index/index_reader.dart';
import '../../../src/store/index_directory.dart';

class URLIndexDirectory implements IndexDirectory {
  final String _url;
  final List<String> _indexFields;
  final List<String> _storedFields;
  URLIndexDirectory(this._url, this._indexFields, this._storedFields);

  @override
  Future<Null> initOutput() async {}

  @override
  Future<Null> outputLiveDocs(Set<int> docIds) {
    throw new Exception('UnsupportedOperationException');
  }

  @override
  Future<Null> outputIndex(String name, Map<String, DocSet> index) {
    throw new Exception('UnsupportedOperationException');
  }

  @override
  Future<Null> outputData(String name, Map<int, DocData> data) {
    throw new Exception('UnsupportedOperationException');
  }

  @override
  void initInput() {}

  @override
  LiveDocsSource inputLiveDocs() {
    return new LiveDocsSource(new URLLiveDocsStreams(_url));
  }

  @override
  IndexSource inputIndex() {
    return new IndexSource(new URLIndexStreams(_indexFields, _url));
  }

  @override
  DataSource inputData() {
    return new DataSource(new URLDataStreams(_storedFields, _url));
  }
}

class URLLiveDocsStreams extends LiveDocsStreams {
  final String _url;
  URLLiveDocsStreams(this._url);
  @override
  Future<Null> readFully(DocSet to) async {
    final String s = await URLUtils._get(_url + '/.liv');
    final Set<int> docIds =
        s.split(',').map((String s) => int.parse(s)).toSet();
    to.update(docIds);
  }
}

class URLIndexStreams extends IndexStreams {
  final List<String> _indexFields;
  final String _url;
  URLIndexStreams(this._indexFields, this._url);
  bool _readFully;

  @override
  Future<Null> readFully(Map<String, Map<String, DocSet>> to) async {
    _readFully = true;
    //for (String field in _indexFields) {
    await Future.forEach(_indexFields, (String field) {
      URLUtils._get(_url + '/$field.idx').then((String str) {
        final List<String> lines = str.split('\n');
        final Map<String, DocSet> invertedIndex = <String, DocSet>{};
        for (String line in lines) {
          final List<String> kv = line.split('=>');
          if (kv.length == 2) {
            final String key = kv[0];
            final Set<int> values =
                kv[1].split(',').map((String s) => int.parse(s)).toSet();
            invertedIndex[key] = new DocSet(values);
          }
        }
        to[field] = invertedIndex;
      });
    }); //}
  }

  @override
  Future<Null> readTerms(
      String field, Map<String, Map<String, DocSet>> to) async {
    if (!_readFully) await readFully(to);
  }

  @override
  Future<Null> readPostings(
      String field, String term, Map<String, Map<String, DocSet>> to) async {
    if (!_readFully) await readFully(to);
  }
}

class URLDataStreams extends DataStreams {
  final List<String> _storedFields;
  final String _url;
  URLDataStreams(this._storedFields, this._url);
  bool _readFully;

  @override
  Future<Null> readFully(Map<String, Map<int, DocData>> to) async {
    _readFully = true;
    //for (String field in _storedFields) {
    await Future.forEach(_storedFields, (String field) {
      URLUtils._get(_url + '/$field.data').then((String str) {
        final List<String> lines = str.split('\n');
        final Map<int, DocData> storedData = <int, DocData>{};
        for (String line in lines) {
          final List<String> kv = line.split('=>');
          if (kv.length == 2) {
            final int key = int.parse(kv[0]);
            final List<String> values = kv[1].split('\$');
            storedData[key] = new DocData(values);
          }
        }
        to[field] = storedData;
      });
    }); //}
  }

  @override
  Future<Null> readFields(Map<String, Map<int, DocData>> to) async {
    if (!_readFully) await readFully(to);
  }

  @override
  Future<Null> readValues(
      String field, int docId, Map<String, Map<int, DocData>> to) async {
    if (!_readFully) await readFully(to);
  }
}

class URLUtils {
  static Future<String> _get(String url) async {
    return HttpRequest.getString(url);
  }
}
