// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:async';

import '../../src/index/index_reader.dart';

/// All i/o is through this API.
abstract class IndexDirectory {
  Future<Null> initOutput();
  Future<Null> outputLiveDocs(Set<int> docIds);
  Future<Null> outputIndex(String name, Map<String, DocSet> index);
  Future<Null> outputData(String name, Map<int, DocData> data);
  void initInput();
  LiveDocsSource inputLiveDocs();
  IndexSource inputIndex();
  DataSource inputData();
}

// ignore: one_member_abstracts
abstract class LiveDocsStreams {
  Future<Null> readFully(DocSet to);
}

abstract class IndexStreams {
  Future<Null> readFully(Map<String, Map<String, DocSet>> to);
  Future<Null> readTerms(String field, Map<String, Map<String, DocSet>> to);
  Future<Null> readPostings(
      String field, String term, Map<String, Map<String, DocSet>> to);
}

abstract class DataStreams {
  Future<Null> readFully(Map<String, Map<int, DocData>> to);
  Future<Null> readFields(Map<String, Map<int, DocData>> to);
  Future<Null> readValues(
      String field, int docId, Map<String, Map<int, DocData>> to);
}

class LiveDocsSource {
  final LiveDocsStreams _streams;
  LiveDocsSource(this._streams);
  Future<DocSet> read(DocSet to) async {
    await _streams.readFully(to);
    return to;
  }
}

class IndexSource {
  final IndexStreams _streams;
  IndexSource(this._streams);
  Future<Map<String, Map<String, DocSet>>> read(
      Map<String, Map<String, DocSet>> to) async {
    await _streams.readFully(to);
    return to;
  }

  Future<Map<String, Map<String, DocSet>>> terms(
      String field, Map<String, Map<String, DocSet>> to) async {
    await _streams.readTerms(field, to);
    return to;
  }

  Future<Map<String, Map<String, DocSet>>> postings(
      String field, String term, Map<String, Map<String, DocSet>> to) async {
    await _streams.readPostings(field, term, to);
    return to;
  }
}

class DataSource {
  DataStreams _streams;
  DataSource(this._streams);
  Future<Map<String, Map<int, DocData>>> read(
      Map<String, Map<int, DocData>> to) async {
    await _streams.readFully(to);
    return to;
  }

  Future<Map<String, Map<int, DocData>>> fields(
      Map<String, Map<int, DocData>> to) async {
    await _streams.readFields(to);
    return to;
  }

  Future<Map<String, Map<int, DocData>>> values(
      String field, int docId, Map<String, Map<int, DocData>> to) async {
    await _streams.readValues(field, docId, to);
    return to;
  }
}
