// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'dart:io';

import 'package:ducene/document.dart';
import 'package:ducene/index.dart';
import 'package:ducene/search.dart';
import 'package:ducene/store.dart';
import 'package:ducene/store_sembast.dart';
import 'package:test/test.dart';

void main() {
  group('A group of directory', () {
    SembastIndexDirectory directory;
    final Directory dir = new Directory('test-dir');

    setUp(() {
      if (dir.existsSync()) dir.deleteSync(recursive: true);
      if (!dir.existsSync()) dir.createSync(recursive: true);
      directory =
          new SembastIndexDirectory("test-dir", <String>["id"], <String>["id"]);
    });

    tearDown(() {
      if (dir.existsSync()) dir.deleteSync(recursive: true);
    });

    test('test 001', () async {
      final IndexWriter writer = new IndexWriter(directory);
      final List<Document> docs = <Document>[];
      for (int i = 0; i < 10; i++) {
        final Document doc = new Document();
        doc.append("id", i.toString());
        docs.add(doc);
      }
      await writer.write(docs);

      final IndexReader reader = await DirectoryReader.open(directory);
      final IndexSearcher searcher = new IndexSearcher(reader);

      expect(await searcher.count(new BoolQuery().append('id', '5')), 1);
      expect(
          await searcher
              .count(new BoolQuery().addQuery(new MatchAllDocsQuery())),
          10);
    });
  });
}
